/*****************************************************************************
* Copyright (c) Future Technology Devices International 2014
* propriety of Future Technology devices International.
*
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by FTDI. 
* FTDI accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file                           Receiver_Enhanced_Flow_Ctl.ino
* @brief                          Sketch to demonstrate enhanced RS485 functionality.
								  Tested platform version: Arduino 1.0.4 and later
* @version                        1.0.0
* @date                           2014/10/01
*
*/


/* Arduino standard includes */
#include "SPI.h"
#include "Wire.h"


#include "FT_VI800A_N485U.h"

#define AUTO_ADDRESS_DETECTION

FT_VI800A VI800A;

uint8_t DevINT = 0;

char outgoingByte=0;  char polling = 0; char incomingByte=0; char command; char incommingData;
uint8_t displayCharacters=1, LSRstat,FIFOStatus, firstTime=1, allInterrupts, bytesInFIFO=0;
char g_var = 'A';
volatile int InterruptOccured=0;
char AddressByte = 'q';

void interruptHasHappened(void);
void CheckInterruptStatus(uint8_t status);

void interruptHasHappened(void){
	InterruptOccured = 1;
}

void CheckInterruptStatus(uint8_t status){
	if(status == FT_VI800A_No_Interrupts)
		return;
	
	if(status == FT_VI800A_Receiver_Time_Out_Interrupt){
			Serial.println("Stale Interrupt.");
			while(VI800A.GetReceiveFIFOFillLevel()){
				Serial.println((char)VI800A.Read());
			}
	}
		
	else if(status == FT_VI800A_RHR_Interrupt){
			Serial.println("Receive FIFO trigger level Interrupt: ");
			while(VI800A.GetReceiveFIFOFillLevel()){	
				Serial.println((char)VI800A.Read());
			}
	}
			
	else if(status == FT_VI800A_Received_Xoff_Signal){
			Serial.println("XOFF Special Character Detected Interrupt.");
			Serial.print("RS485 Address Character: ");
			Serial.println((char)VI800A.Read());
#ifndef AUTO_ADDRESS_DETECTION
			if(VI800A.Read() == AddressByte)
				VI800A.SetReceiver(FT_VI800A_Enable);
			else
				VI800A.SetReceiver(FT_VI800A_Disable);
#endif
	}
}

void setup()
{
	pinMode(FT_N485U_INT,INPUT);
	digitalWrite(FT_N485U_INT,HIGH);
	
	
	/* Initialize serial print related functionality */
	Serial.begin(9600);
    Serial.println("\n\r\n\r-----------Receiver_Enhanced_Flow_Ctl.ino--------------");
	

	VI800A.Init(FT_N485U_CS, 9600,FT_VI800A_Forced_Parity_Zero, FT_VI800A_One_Stop_Bit, FT_VI800A_Eight_Data_Bits);
	
#ifdef AUTO_ADDRESS_DETECTION
	VI800A.SetFlowControl(FT_VI800A_RS485_Flow,FT_VI800A_Auto_RS485_Address_Detection_Mode,AddressByte);
#else	
	VI800A.SetFlowControl(FT_VI800A_RS485_Flow,FT_VI800A_RS485_Multidrop_Mode,AddressByte);
#endif
	
	VI800A.EnableFIFO(FT_VI800A_Enable);
	VI800A.clearFIFO();
	
	VI800A.SetInterrupts(FT_VI800A_Xoff_Interrupt|FT_VI800A_Receive_Holding_Register_Interrupt);
	
	VI800A.SetFIFOInterruptTriggerLevel(8,8);	
	attachInterrupt(DevINT,interruptHasHappened,FALLING);
}


void loop()
{
	if (Serial.available() > 0){
		VI800A.Write(Serial.read());
	}
	

	
	if(InterruptOccured==1){
		CheckInterruptStatus(VI800A.GetInterrupts());
		InterruptOccured=0;
	}
}



























































